#!/bin/bash
#Autor: José Antonio Serrano Rosso
#Descripción: Sistema de automatización de workarounds y ejecucion de scripts ocasionales (SAWESO)

#Variables globales
EJECUCION=$1
RUTACONFIG="/etc/saweso"
declare -a DATOS_PLANTILLA
HILOS=10
PUERTO_PROXY=0

#Directorios de trabajo
DIR_REGISTRO="/var/log/saweso"
DIR_RESULTADOS="$DIR_REGISTRO/$EJECUCION/Resultados"

#Obtenemos la fecha en formatos dd-mm-yyyy y la hora en formato hh:mm:ss
FECHA="$(date +"%d-%m-%Y")"
HORA="$(date +"%H_%M_%S")"

#Parametros leidos desde fichero de configuracion
SAWESO_BD=`grep --only-matching --perl-regex "(?<=schemaBD\ =).*" $RUTACONFIG/saweso.conf | tr -d " "`
SAWESO_USERBD=`grep --only-matching --perl-regex "(?<=usernameBD\ =).*" $RUTACONFIG/saweso.conf | tr -d " "`
SAWESO_PASSBD=`grep --only-matching --perl-regex "(?<=passwordBD\ =).*" $RUTACONFIG/saweso.conf | tr -d " "`

#Parametros de configuracion del cliente ssh
ssh_setup1=-o"ConnectTimeout 10"	
ssh_setup2=-o"BatchMode yes"		
ssh_setup3=-o"SetupTimeOut 10"
ssh_setup4=-o"CheckHostIP no"
ssh_setup5=-o"StrictHostKeyChecking no"
ssh_setup6=-o"UserKnownHostsFile /dev/null"
ssh_setup_SERVIDOR=-i"$RUTACONFIG/`grep --only-matching --perl-regex "(?<=ssh_setup_SERVIDOR\ =).*" $RUTACONFIG/saweso.conf | tr -d " "`"
ssh_setup_CLIENTE=-i"$RUTACONFIG/`grep --only-matching --perl-regex "(?<=ssh_setup_CLIENTE\ =).*" $RUTACONFIG/saweso.conf | tr -d " "`"


################################################### DECLARACION DE FUNCIONES ###################################################

#NOMBRE: isInteger - comprueba si un número es entero
#SINOPSIS: isInteger <numero>
#ENTRADA: $1 - numero a comprobar
#SALIDA: devuelve 1 = numero NO entero; 0 = numero entero
isInteger()
{
  local x=$1
  case $x in
    *[!0-9+-]*|?*[-+]*|""|-|+)
      return 1 ;;
    *)
      return 0 ;;
  esac
}


#NOMBRE: gestorBD - registra las consultas en la base de datos MySQL
#SINOPSIS: gestorBD <consulta_sql>
#ENTRADA: $1 - consulta sql a ejecutar
#SALIDA: produce la salida del script con errores (exit 1) si se produce algún fallo al ejecutar la consulta
gestorBD(){
local consulta_sql=$1

if [ -n "$consulta_sql" ]; then
	mysql -u"$SAWESO_USERBD" -p"$SAWESO_PASSBD" -D "$SAWESO_BD" -e "$consulta_sql;" -N
	if [ "$?" != "0" ];then
		logger "SAWESO.SH (FATAL EXIT): No se ha podido ejecutar la consulta $sql. La base de datos MySQL puede estar parada"
        	exit 1
	fi
else
	logger "SAWESO.SH (FATAL EXIT): No se especificado ninguna consulta SQL a ejecutar"
	exit 1
fi
}


#NOMBRE: log - registra el mensaje especificado en el log de la aplicacion
#SINOPSIS: log <mensaje>
#ENTRADA: $1 - mensaje de texto a registrar
log()
{
local mensaje=$1

if [ -n "$mensaje" ]; then
	gestorBD "INSERT INTO log VALUES (DEFAULT, '$EJECUCION', NOW(), '$mensaje')"
else
	logger "SAWESO.SH: No se ha podido registrar el mensaje en el log de la aplicacion. No se ha especificado código de ejecucion o el mensaje esta vacío"
fi
}


#NOMBRE: ComprobarEntornoEjecucion - comprueba el sistema para verificar la correcta ejecución de saweso
#SINOPSIS: ComprobarEntornoEjecucion
#SALIDA: produce la salida del script con errores (exit 1) en caso de encontrar alguna condicion incumplida
ComprobarEntornoEjecucion()
{
#Comprueba si hemos especificado un codigo de ejecucion entero al lanzar saweso.sh
isInteger $EJECUCION
local check_entero=$?
if [ -z $EJECUCION ] || [ $check_entero -eq 1 ];then
	logger "SAWESO.SH (FATAL EXIT): no se ha especificado un codigo de ejecucion ($EJECUCION) correcto ($check_entero) al ejecutar el script"
	exit 1
fi

#Comprueba si el usuario es root
#local usuario=`whoami`
#if [ -z $usuario ] || [ "$usuario" != "root" ];then
#	logger "SAWESO.SH (FATAL EXIT): El script $0 debe ser ejecutado por el usuario root"
#	exit 1
#fi

#Comprueba si tenemos permiso de escritura en el directorio de registro
if [ ! -d "$RUTACONFIG" ] || [ ! -r "$RUTACONFIG" ] || [ ! -w "$RUTACONFIG" ]; then
	logger "SAWESO.SH (FATAL EXIT): No se puede leer o escribir en el directorio de configuracion $RUTACONFIG"
	exit 1	
else
        mkdir -p $DIR_RESULTADOS 2> /dev/null
fi

#Comprueba si existe la carpeta de scripts de saweso
if [ ! -d $RUTACONFIG/PlantillasEjecucion ];then
        logger "SAWESO.SH (FATAL EXIT): No existe el directorio de scripts ($RUTACONFIG/PlantillasEjecucion) de saweso. Reinstale SAWESO"
	exit 1
fi

#Comprueba si tenemos permiso de escritura en el directorio de registro
if [ ! -d "$DIR_REGISTRO" ] || [ ! -r "$DIR_REGISTRO" ] || [ ! -w "$DIR_REGISTRO" ]; then
	logger "SAWESO.SH (FATAL EXIT): No se puede leer o escribir en el directorio de registro $DIR_REGISTRO"
	exit 1	
else
        mkdir -p $DIR_RESULTADOS 2> /dev/null
fi

#Comprueba si saweso.sh esta en la ubicación correcta
if [ ! -e "/usr/bin/saweso.sh" ] || [ ! -x "/usr/bin/saweso.sh" ] || [ ! -r "/usr/bin/saweso.sh" ];then
	logger "SAWESO.SH (FATAL EXIT): No se puede localizar el script /usr/bin/saweso.sh o no tiene los permisos correctos. Reinstale SAWESO"
	exit 1
fi

#Comprueba si tenemos comunicacion con la BD registrando un mensaje de log inicial. En caso de error la funcion mysql() provocara la salida exit 1
log "################################################################################################################"
log "##### SAWESO.SH (`whoami`): INICIO DE SAWESO $FECHA.$HORA #####"
log "##### SAWESO.SH (`whoami`): ComprobarEntornoEjecucion TEST OK #####"
}


#NOMBRE: 
#SINOPSIS: 
ObtenerDatosIniciales()
{ 
local nombrePlantilla=`mysql -u"$SAWESO_USERBD" -p"$SAWESO_PASSBD" -D "$SAWESO_BD" -e "select plantilla from ejecucion where codigo='$EJECUCION';" -N`
local tipo=`mysql -u"$SAWESO_USERBD" -p"$SAWESO_PASSBD" -D "$SAWESO_BD" -e "select tipo from plantilla where nombre='$nombrePlantilla';" -N`
local script=`find $RUTACONFIG -name $nombrePlantilla`
local usuario=`mysql -u"$SAWESO_USERBD" -p"$SAWESO_PASSBD" -D "$SAWESO_BD" -e "select usuario from ejecucion where codigo='$EJECUCION';" -N`
local email=`mysql -u"$SAWESO_USERBD" -p"$SAWESO_PASSBD" -D "$SAWESO_BD" -e "select email from usuario where nombre='$usuario';" -N`
local IPF0=`mysql -u"$SAWESO_USERBD" -p"$SAWESO_PASSBD" -D "$SAWESO_BD" -e "select ip_firewall from ejecucion where codigo='$EJECUCION';" -N`

DATOS_PLANTILLA=([1]=$EJECUCION [2]=$tipo [3]=$IPF0 [4]=$script [5]=$email)

log "##### SAWESO.SH (`whoami`): DATOS ARRAY: ${DATOS_PLANTILLA[*]} #####"
}


#NOMBRE: Marcar - establece la etiqueta "EJECUTADO", "COPIADO" o "FIN" sobre el equipo especificado
#SINOPSIS: Marcar <ip> <estado>
#ENTRADA: $1 - direccion ip; $2 - estado a establecer ("ejecutado", "copiado", "finalizado")
Marcar()
{
local ip=`echo "$1" | tr -d " "`
local estado="$2"
local hilo="$3"

case "$estado" in
        "asignado")
		gestorBD "UPDATE ejecucionhost SET estado='ASIGNADO $hilo' WHERE ejecucion='$EJECUCION' AND host='$ip'"
		log "Equipo $ip asignado al hilo $hilo"
	;;
        "noejecutado")
		gestorBD "UPDATE ejecucionhost SET estado='NO EJECUTADO $hilo' WHERE ejecucion='$EJECUCION' AND host='$ip'"
		log "HILO $hilo: Equipo $ip marcado como NO ejecutado"
	;;
        "ejecutado")
		gestorBD "UPDATE ejecucionhost SET estado='EJECUTADO $hilo' WHERE ejecucion='$EJECUCION' AND host='$ip'"
		log "HILO $hilo: Equipo $ip marcado como ejecutado"
	;;
	"nofinalizado")
		gestorBD "UPDATE ejecucionhost SET estado='NO FIN $hilo' WHERE ejecucion='$EJECUCION' AND host='$ip'"
		log "HILO $hilo: Equipo $ip marcado como NO finzalizado"
	;;
	"finalizado")
		gestorBD "UPDATE ejecucionhost SET estado='FIN' WHERE ejecucion='$EJECUCION' AND host='$ip'"
		log "HILO $hilo: Equipo $ip marcado como finalizado"
	;;
esac
}


#NOMBRE: SSH - Establece conexión ssh y ejecuta el script seleccionado sobre el servidor indicado
#SINOPSIS: SSH <ip_servidor> <script a ejecutar>
#ENTRADA: $1 - ip del servidor; $2 - script a ejecutar en el servidor;
#SALIDA: devuelve 1 = proceso CrearTunel-CopiarScript-EjecutarScript finalizado con éxito; 0 = proceso CrearTunel-CopiarScript-EjecutarScript no finalizado
SSH()
{
local hilo=$1
local script="${DATOS_PLANTILLA[4]}"
local script_inicial="inicio.sh"
local nombre_script=`basename $script`
local FAIL
local ip=`mysql -u"$SAWESO_USERBD" -p"$SAWESO_PASSBD" -D "$SAWESO_BD" -e "select host from ejecucionhost where ejecucion='$EJECUCION' AND estado='ASIGNADO $hilo' LIMIT 1;" -N`

log "HILO $hilo: ejecucion iniciada ..."
until [ -z $ip ];do
	FAIL=0
	log "HILO $hilo: procesando host $ip"
	#Copiamos script al servidor
	log "HILO $hilo: Copiamos script al servidor $ip"
	scp -q "$ssh_setup_SERVIDOR" "$ssh_setup1" "$ssh_setup2" "$ssh_setup3" "$ssh_setup4" "$ssh_setup5" "$ssh_setup6" -p $script root@$ip:/tmp 2> /dev/null
	if [ "$?" == "0" ];then
		#Ejecutar script en el servidor
		if [ `echo $nombre_script | grep ".sh"` ];then
        		log "HILO $hilo: Ejecutamos el script /tmp/$nombre_script con codigo de ejecucion $EJECUCION en el servidor $ip"
			ssh -q -f "$ssh_setup_SERVIDOR" "$ssh_setup1" "$ssh_setup2" "$ssh_setup3" "$ssh_setup4" "$ssh_setup5" "$ssh_setup6" -C root@$ip "bash /tmp/$nombre_script $EJECUCION $ip &" 2> /dev/null
			[ "$?" != "0" ] && FAIL=1			
		elif [ `echo $nombre_script | grep ".tar.gz"` ];then
			log "HILO $hilo: Descomprimimos el tar.gz /tmp/$nombre_script en el servidor $ip"
        		ssh -q -f "$ssh_setup_SERVIDOR" "$ssh_setup1" "$ssh_setup2" "$ssh_setup3" "$ssh_setup4" "$ssh_setup5" "$ssh_setup6" -C root@$ip "cd /tmp && tar -xzpf /tmp/$nombre_script &" 2> /dev/null
			if [ "$?" == "0" ];then
				log "HILO $hilo: Ejecutamos el script /tmp/$nombre_script con codigo de ejecucion $EJECUCION en el servidor $ip"
        			ssh -q -f "$ssh_setup_SERVIDOR" "$ssh_setup1" "$ssh_setup2" "$ssh_setup3" "$ssh_setup4" "$ssh_setup5" "$ssh_setup6" -C root@$ip "cd /tmp && bash /tmp/$script_inicial $EJECUCION $ip &" 2> /dev/null
				[ "$?" != "0" ] && FAIL=1
			else
				FAIL=1			
			fi
		else
        		log "HILO $hilo: ERROR: No se ha localizado el nombre del script: $nombre_script"
			FAIL=1
        		exit 1
		fi
	else
		FAIL=1
	fi	

	if [ "$FAIL" == "1" ];then
	        log "HILO $hilo: Proceso SSH-CopiarScript-EjecutarScript con $ip no completado con éxito"
		Marcar $ip noejecutado $hilo
	else
	        log "HILO $hilo: Proceso SSH-CopiarScript-EjecutarScript con $ip finalizado con éxito"
		Marcar $ip ejecutado $hilo
	fi

	ip=`mysql -u"$SAWESO_USERBD" -p"$SAWESO_PASSBD" -D "$SAWESO_BD" -e "select host from ejecucionhost where ejecucion='$EJECUCION' AND estado='ASIGNADO $hilo' LIMIT 1;" -N`
done

log "HILO $hilo: ejecucion finalizada"
RecogerResultados $hilo

return
}


#NOMBRE: CheckSSH - comprueba la correcta conectividad por ssh con el host especificado 
#SINOPSIS: CheckSSHServidor <ip_servidor>
#ENTRADA: $1 - direccion ip del host
#SALIDA: devuelve 0 = conectividad ssh ok; 1 = conectividad ssh down
CheckSSHServidor()
{
local ip=$1

#Comprobamos conectividad por ssh
log "Comprobando conectividad ssh con el SERVIDOR $ip"
ssh -q -f "$ssh_setup_SERVIDOR" "$ssh_setup1" "$ssh_setup2" "$ssh_setup3" "$ssh_setup4" "$ssh_setup5" "$ssh_setup6" root@$ip "touch /tmp/saweso.link.ok" > /dev/null
if [ "$?" != "0" ];then
	log "No se puede establecer conexion ssh con SERVIDOR $ip"
	return 1
else
        log "Conexion ssh establecida con SERVIDOR $ip"
	return 0
fi
}


#NOMBRE: Tunneling - Crea un túnel ssh y ejecuta el script seleccionado sobre el equipo indicado
#SINOPSIS: Tunneling <ip_equipo> <script a ejecutar> <ip_servidor>
#SALIDA: devuelve 1 = proceso CrearTunel-CopiarScript-EjecutarScript finalizado con éxito; 0 = proceso CrearTunel-CopiarScript-EjecutarScript no finalizado
Tunneling()
{
local hilo=$1
local script="${DATOS_PLANTILLA[4]}"
local script_inicial="inicio.sh"
local nombre_script=`basename $script`
local servidor="${DATOS_PLANTILLA[3]}"
local FAIL
local ip=`mysql -u"$SAWESO_USERBD" -p"$SAWESO_PASSBD" -D "$SAWESO_BD" -e "select host from ejecucionhost where ejecucion='$EJECUCION' AND estado='ASIGNADO $hilo' LIMIT 1;" -N`
local port=$PUERTO_PROXY

log "HILO $hilo: ejecucion tunneling iniciada ..."
until [ -z $ip ];do
	FAIL=0
	log "HILO $hilo: procesando host $ip"
	#Copiamos script al servidor
	log "HILO $hilo: Copiamos script al equipo $ip utilizando SSH tunel en puerto $port"
	scp -q "$ssh_setup_CLIENTE" "$ssh_setup1" "$ssh_setup2" "$ssh_setup3" "$ssh_setup4" "$ssh_setup5" "$ssh_setup6" -o "ProxyCommand /bin/nc.openbsd -x localhost:$port %h %p" -p $script root@$ip:/tmp 2> /dev/null
	if [ "$?" == "0" ];then
		#Ejecutar script en el equipo
		if [ `echo $nombre_script | grep ".sh"` ];then
       			log "HILO $hilo: Ejecutamos el script /tmp/$nombre_script con codigo de ejecucion $EJECUCION en el equipo $ip utilizando SSH tunel en puerto $port"
			ssh -lroot -q -f -C "$ssh_setup_CLIENTE" "$ssh_setup1" "$ssh_setup2" "$ssh_setup3" "$ssh_setup4" "$ssh_setup5" "$ssh_setup6" -o "ProxyCommand /bin/nc.openbsd -x localhost:$port %h %p" $ip "bash /tmp/$nombre_script $EJECUCION $ip &" 2> /dev/null
			[ "$?" != "0" ] && FAIL=1			
		elif [ `echo $nombre_script | grep ".tar.gz"` ];then
			log "HILO $hilo: Descomprimimos el tar.gz /tmp/$nombre_script en el equipo $ip utilizando SSH tunel en puerto $port"
			ssh -lroot -q -f -C "$ssh_setup_CLIENTE" "$ssh_setup1" "$ssh_setup2" "$ssh_setup3" "$ssh_setup4" "$ssh_setup5" "$ssh_setup6" -o "ProxyCommand /bin/nc.openbsd -x localhost:$port %h %p" $ip "cd /tmp && tar -xzpf /tmp/$nombre_script &" 2> /dev/null
			if [ "$?" == "0" ];then
				log "HILO $hilo: Ejecutamos el script /tmp/$nombre_script con codigo de ejecucion $EJECUCION en el equipo $ip utilizando SSH tunel en puerto $port"
       				ssh -lroot -q -f -C "$ssh_setup_CLIENTE" "$ssh_setup1" "$ssh_setup2" "$ssh_setup3" "$ssh_setup4" "$ssh_setup5" "$ssh_setup6" -o "ProxyCommand /bin/nc.openbsd -x localhost:$port %h %p" $ip "cd /tmp && bash /tmp/$script_inicial $EJECUCION $ip &" 2> /dev/null
				[ "$?" != "0" ] && FAIL=1
			else
				FAIL=1			
			fi
		else
       			log "HILO $hilo: ERROR: No se ha localizado el nombre del script: $nombre_script"
			FAIL=1
       			exit 1
		fi
	else
		FAIL=1
	fi	

	if [ "$FAIL" == "1" ];then
        	log "HILO $hilo: Proceso Tunneling-CopiarScript-EjecutarScript con $ip no completado con éxito"
		Marcar $ip noejecutado $hilo
	else
        	log "HILO $hilo: Proceso Tunneling-CopiarScript-EjecutarScript con $ip finalizado con éxito"
		Marcar $ip ejecutado $hilo
	fi

	ip=`mysql -u"$SAWESO_USERBD" -p"$SAWESO_PASSBD" -D "$SAWESO_BD" -e "select host from ejecucionhost where ejecucion='$EJECUCION' AND estado='ASIGNADO $hilo' LIMIT 1;" -N`
done
log "HILO $hilo: ejecucion Tunneling finalizada"
RecogerResultados $hilo

return
}


#NOMBRE: CloseTunneling - cierra el tunel abierto en la función Tunneling 
#SINOPSIS: CloseTunneling
CloseTunneling()
{
local port=$PUERTO_PROXY

log "Cerramos el tunel (puerto $port) SSH dynamic port forwarding with SOCKS creado para la ejecucion en equipos clientes"
pid_tunel=`netstat -tanpe | grep -i ssh | grep $port$ | awk '{print $9}' | awk -F"/" '{print $1}'`
kill $pid_tunel 2> /dev/null
}


#NOMBRE: RecogerResultados - recoge los resultados (archivador .tar.gz) sobre todos los equipos donde ha finalizado la fase de ejecucion
#SINOPSIS: RecogerResultados equipo <ip_equipo> <ip_servidor> ò RecogerResultados servidor <ip_servidor>
#ENTRADA: $1 - tipo de host (equipo o servidor); $2 - direccion ip del host; $3 - direccion ip del servidor en caso de comprobacion sobre equipo
#SALIDA: devuelve 1 = resultados recogidos correctamente; 0 = resultados no recopilados
RecogerResultados()
{
local hilo=$1
local port=$PUERTO_PROXY
local FAIL
local ip=`mysql -u"$SAWESO_USERBD" -p"$SAWESO_PASSBD" -D "$SAWESO_BD" -e "select host from ejecucionhost where ejecucion='$EJECUCION' AND estado='EJECUTADO $hilo' LIMIT 1;" -N`

log "HILO $hilo: recogida de resultados iniciada ..."
until [ -z $ip ];do
	FAIL=0
	case "${DATOS_PLANTILLA[2]}" in
        	"equipo")
        	#Recogemos los resultados del equipo al directorio de resultados
        	log "Recogemos los resultados del equipo $ip al directorio de resultados"
		scp -q "$ssh_setup_CLIENTE" "$ssh_setup1" "$ssh_setup2" "$ssh_setup3" "$ssh_setup4" "$ssh_setup5" "$ssh_setup6" -o "ProxyCommand /bin/nc.openbsd -x localhost:$port %h %p" -p root@$ip:/tmp/$EJECUCION/*.tar.gz $DIR_RESULTADOS 2> /dev/null
		if [ "$?" == "0"  ];then
        	        log "Resultados del equipo $ip almacenados en $DIR_RESULTADOS"
			ssh -lroot -q -f -C "$ssh_setup_CLIENTE" "$ssh_setup1" "$ssh_setup2" "$ssh_setup3" "$ssh_setup4" "$ssh_setup5" "$ssh_setup6" -o "ProxyCommand /bin/nc.openbsd -x localhost:$port %h %p" $ip "rm -rf /tmp/saweso.* && rm -rf /tmp/$EJECUCION &" 2> /dev/null
			log "Eliminamos los ficheros temporales /tmp/$EJECUCION en $ip"
        	        log "Eliminamos el script ejecutado de /tmp/ en $ip"
		else
			FAIL=1			
        	fi
		;;
		"servidor")
        	#Recogemos los resultados del servidor al directorio de resultados
        	log "Recogemos los resultados del equipo $ip al directorio de resultados"
        	scp -q "$ssh_setup_SERVIDOR" "$ssh_setup1" "$ssh_setup2" "$ssh_setup3" "$ssh_setup4" "$ssh_setup5" "$ssh_setup6" -p root@$ip:/tmp/$EJECUCION/*.tar.gz $DIR_RESULTADOS 2> /dev/null
        	if [ "$?" == "0"  ];then
        	        log "Resultados del equipo $ip almacenados en $DIR_RESULTADOS"
        	        ssh -q -f "$ssh_setup_SERVIDOR" "$ssh_setup1" "$ssh_setup2" "$ssh_setup3" "$ssh_setup4" "$ssh_setup5" "$ssh_setup6" -C root@$ip "rm -rf /tmp/$EJECUCION &" 2> /dev/null
			log "Eliminamos los ficheros temporales /tmp/$EJECUCION en $ip"
		else
			FAIL=1			
        	fi
		;;
	esac	

	if [ "$FAIL" == "1" ];then
        	log "Los resultados de $ip no pueden ser recopilados"
		Marcar $ip nofinalizado $hilo
	else
        	log "Resultados de $ip recopilados con éxito en $DIR_RESULTADOS" 
		Marcar $ip finalizado $hilo
	fi

	ip=`mysql -u"$SAWESO_USERBD" -p"$SAWESO_PASSBD" -D "$SAWESO_BD" -e "select host from ejecucionhost where ejecucion='$EJECUCION' AND estado='EJECUTADO $hilo' LIMIT 1;" -N`
done

log "HILO $hilo: recogida de resultados finalizada"
}


#NOMBRE: MultiHilo - Abre "n" hilos del script saweso.sh
#SINOPSIS: MultiHilo
MultiHilo()
{
log "Ejecutando MultiHilo ..."

#Restablecemos estados para continuar con la ejecucion de los host aún no ejecutados
log "Restablecemos estado INICIO para continuar con la ejecucion de los host aún no ejecutados"
gestorBD "UPDATE ejecucionhost SET estado='INICIO' WHERE ejecucion='$EJECUCION' AND estado LIKE '%NO EJECUTADO%'"

#Restablecemos estados para continuar con la recogida de resultados en los host pendientes
log "Restablecemos estado EJECUTADO para continuar con la recogida de resultados en los host pendientes"
gestorBD "UPDATE ejecucionhost SET estado=REPLACE(estado, 'NO FIN', 'EJECUTADO') WHERE estado LIKE '%NO FIN%'"


local host=`mysql -u"$SAWESO_USERBD" -p"$SAWESO_PASSBD" -D "$SAWESO_BD" -e "select host from ejecucionhost where ejecucion='$EJECUCION' AND estado='INICIO' LIMIT 1;" -N`
local count=1
local servidor="${DATOS_PLANTILLA[3]}"

log "Repartiendo direcciones ip para cada proceso hijo ..."
until [ -z $host ];do
	[ $count -gt $HILOS ] && count=1
	Marcar $host asignado $count
	host=`mysql -u"$SAWESO_USERBD" -p"$SAWESO_PASSBD" -D "$SAWESO_BD" -e "select host from ejecucionhost where ejecucion='$EJECUCION' AND estado='INICIO' LIMIT 1;" -N`
	((++count))	
done

log "Lanzando $HILOS hilos para la ejecucion código ${DATOS_PLANTILLA[1]} de tipo ${DATOS_PLANTILLA[2]}"
case "${DATOS_PLANTILLA[2]}" in
        "equipo")
		CheckSSHServidor $servidor
		if [ "$?" == "0" ];then
			#Buscamos un puerto libre para crear un proxy	
			log "Buscando puerto libre para la creacion de un proxy SOCKS"
			until [ $port -gt 30000 ] && [ $port -lt 65535 ];do
				port=$[$RANDOM+30000]
				num=`netstat -ta | grep $port | wc -l`
				if [ $num -ne 0 ];then
					port=0
				fi
			done
			log "Puerto libre encontrado: $port"
			#Exportamos el puerto para su uso global
			PUERTO_PROXY=$port
			#Creamos el tunel SSH 
			log "Creando SSH dynamic port forwarding with SOCKS"
			autossh -f -C "$ssh_setup_SERVIDOR" "$ssh_setup1" "$ssh_setup2" "$ssh_setup3" "$ssh_setup4" "$ssh_setup5" "$ssh_setup6" -N -D localhost:$port root@$servidor > /dev/null
			log "SSH dynamic port forwarding with SOCKS creado con éxito"
			for ((j=1;j<=$HILOS;j=j+1));do		
				Tunneling $j &
				log "Ejecutando proceso hijo número $j de $HILOS"
			done
		else
			log "Sin conexión con el servidor pasarela ($servidor). Imposible establecer SSH dynamic port forwarding with SOCKS"
			log "Fase de recogida de resultados cancelada"
		fi
	;;
	"servidor")
		for ((j=1;j<=$HILOS;j=j+1));do		
			SSH $j &
			log "Ejecutando proceso hijo número $j de $HILOS"
		done
	;;
esac
}


#NOMBRE: CheckFinish - Comprueba si ha finalizado la ejecucion y recogida de resultados sobre todas las ips del listado
#SINOPSIS: CheckFinish
CheckFinish()
{
local finish=`mysql -u"$SAWESO_USERBD" -p"$SAWESO_PASSBD" -D "$SAWESO_BD" -e "SELECT COUNT(*) AS COUNT FROM ejecucionhost WHERE ejecucion='$EJECUCION' AND estado REGEXP '^INICIO$|^ASIGNADO|^EJECUTADO';" -N`

until [ $finish -eq 0 ];do
	sleep 30
	finish=`mysql -u"$SAWESO_USERBD" -p"$SAWESO_PASSBD" -D "$SAWESO_BD" -e "SELECT COUNT(*) AS COUNT FROM ejecucionhost WHERE ejecucion='$EJECUCION' AND estado REGEXP '^INICIO$|^ASIGNADO|^EJECUTADO';" -N`
done

log "Ejecucion y recogida de resultados finalizada sobre todos los equipos de la ejecucion"
}


#NOMBRE: GenerarInforme - genera un informe en texto plano del estado actual de la ejecucion de saweso
#SINOPSIS: GenerarInforme
#SALIDA: almacena el informe generado en $DIR_REGISTRO/$EJECUCION/$FECHA.$HORA.$INFORME
GenerarInforme()
{
local INFORME="informe.txt"
local pendientes=`mysql -u"$SAWESO_USERBD" -p"$SAWESO_PASSBD" -D "$SAWESO_BD" -e "SELECT COUNT(*) AS COUNT FROM ejecucionhost WHERE ejecucion='$EJECUCION' AND estado REGEXP '^NO EJECUTADO';" -N`
local ejecutadas=`mysql -u"$SAWESO_USERBD" -p"$SAWESO_PASSBD" -D "$SAWESO_BD" -e "SELECT COUNT(*) AS COUNT FROM ejecucionhost WHERE ejecucion='$EJECUCION' AND estado REGEXP '^FIN$|^NO FIN|^EJECUTADO';" -N`
local finalizadas=`mysql -u"$SAWESO_USERBD" -p"$SAWESO_PASSBD" -D "$SAWESO_BD" -e "SELECT COUNT(*) AS COUNT FROM ejecucionhost WHERE ejecucion='$EJECUCION' AND estado REGEXP '^FIN$';" -N`
local ipaux
local nombreficheroaux
local ultimoinforme
local linea

ultimoinforme=`ls $DIR_REGISTRO/$EJECUCION | grep ".$INFORME$"`
if [ $? -eq 0 ];then
	iteracion=`ls $DIR_REGISTRO/$EJECUCION | grep "$INFORME" | wc -l`
	cp -a $DIR_REGISTRO/$EJECUCION/$ultimoinforme $DIR_REGISTRO/$EJECUCION/$ultimoinforme.iteracion$iteracion
	mv $DIR_REGISTRO/$EJECUCION/$ultimoinforme $DIR_REGISTRO/$EJECUCION/$FECHA.$HORA.$INFORME
	#RESUMEN ESTADO DE LA EJECUCION	
	#Actualizamos los datos anteriores con el estado actual
	linea=`grep "Numero de equipos pendientes de ejecucion:" $DIR_REGISTRO/$EJECUCION/$FECHA.$HORA.$INFORME | awk -F": " '{print $2}'`
	sed -i -e 's/'Numero\ de\ equipos\ pendientes\ de\ ejecucion:\ $linea'/'Numero\ de\ equipos\ pendientes\ de\ ejecucion:\ $pendientes'/' $DIR_REGISTRO/$EJECUCION/$FECHA.$HORA.$INFORME
	linea=`grep "Numero de equipos donde se ha ejecutado el script:" $DIR_REGISTRO/$EJECUCION/$FECHA.$HORA.$INFORME | awk -F": " '{print $2}'`
	sed -i -e 's/'Numero\ de\ equipos\ donde\ se\ ha\ ejecutado\ el\ script:\ $linea'/'Numero\ de\ equipos\ donde\ se\ ha\ ejecutado\ el\ script:\ $ejecutadas'/' $DIR_REGISTRO/$EJECUCION/$FECHA.$HORA.$INFORME
	linea=`grep "Numero de equipos donde se ha ejecutado el script y se ha descargado el resultado de la ejecucion:" $DIR_REGISTRO/$EJECUCION/$FECHA.$HORA.$INFORME | awk -F": " '{print $2}'`
	sed -i -e 's/'Numero\ de\ equipos\ donde\ se\ ha\ ejecutado\ el\ script\ y\ se\ ha\ descargado\ el\ resultado\ de\ la\ ejecucion:\ $linea'/'Numero\ de\ equipos\ donde\ se\ ha\ ejecutado\ el\ script\ y\ se\ ha\ descargado\ el\ resultado\ de\ la\ ejecucion:\ $finalizadas'/' $DIR_REGISTRO/$EJECUCION/$FECHA.$HORA.$INFORME

	#DETALLES
	#Descomprimimos los resultados
	log "Descomprimiendo resultados ..."
	if [ -d $DIR_RESULTADOS ];then
		cd $DIR_RESULTADOS
		for i in `ls | grep .tar.gz`;do
			nombreficheroaux=`echo $i | sed 's/'.tar.gz'//g'`
			ipaux=`echo $nombreficheroaux | awk -F"." '{print $4"."$5"."$6"."$7}'`
			#echo "DEBUG: nombreficheroaux = $nombreficheroaux - ipaux = $ipaux"
        		tar -xzpf $i 2> /dev/null
			rm -rf $i
			#for j in `ls $nombreficheroaux* | grep -v "$noincluir$" | grep "$incluir$"`;do
			#	echo -e "\nIP: $ipaux" >> $DIR_REGISTRO/$EJECUCION/$FECHA.$HORA.$INFORME
			#	echo "($j):" >> $DIR_REGISTRO/$EJECUCION/$FECHA.$HORA.$INFORME
			#	cat $j >> $DIR_REGISTRO/$EJECUCION/$FECHA.$HORA.$INFORME
			#	echo -e "\n" >> $DIR_REGISTRO/$EJECUCION/$FECHA.$HORA.$INFORME
			#done
			for j in `ls $nombreficheroaux*`;do
				if [ `echo $j | grep "salida.log$"` ];then
					echo -e "\nIP: $ipaux" >> $DIR_REGISTRO/$EJECUCION/$FECHA.$HORA.$INFORME				
					echo "($j):" >> $DIR_REGISTRO/$EJECUCION/$FECHA.$HORA.$INFORME
					cat $j >> $DIR_REGISTRO/$EJECUCION/$FECHA.$HORA.$INFORME
					echo -e "\n" >> $DIR_REGISTRO/$EJECUCION/$FECHA.$HORA.$INFORME
					gestorBD "UPDATE ejecucionhost SET host_informe='$DIR_RESULTADOS/$j' WHERE ejecucion='$EJECUCION' AND host='$ipaux'"
				elif [ `echo $j | grep "saweso.log$"` ];then
					gestorBD "UPDATE ejecucionhost SET host_log='$DIR_RESULTADOS/$j' WHERE ejecucion='$EJECUCION' AND host='$ipaux'"		
				else
					gestorBD "UPDATE ejecucionhost SET host_resultados=CONCAT_WS(';', host_resultados, '$DIR_RESULTADOS/$j') WHERE ejecucion='$EJECUCION' AND host='$ipaux'"		
				fi
			done
		done
		log "Informe en texto plano generado con éxito: $DIR_REGISTRO/$EJECUCION/$FECHA.$HORA.$INFORME"
	else
		log "No existe el directorios de resultados: $DIR_REGISTRO/$EJECUCION/$FECHA.$HORA.$INFORME"
	fi
else
	#Generamos el informe
	log "Generando informe en texto plano ..."

	#CABECERA
	echo -e "\n\t\t\t\t\t EJECUCION DE SAWESO (Fecha: $FECHA Hora: $HORA)" >> $DIR_REGISTRO/$EJECUCION/$FECHA.$HORA.$INFORME
	echo "Codigo de ejecucion: ${DATOS_PLANTILLA[1]}" >> $DIR_REGISTRO/$EJECUCION/$FECHA.$HORA.$INFORME
	echo "Ejecucion sobre: ${DATOS_PLANTILLA[2]}" >> $DIR_REGISTRO/$EJECUCION/$FECHA.$HORA.$INFORME

	#Comprobamos si es una ejecución sobre equipos o sobre servidores
	if [ -n "${DATOS_PLANTILLA[3]}" ];then
		echo "IP Servidor pasarela: ${DATOS_PLANTILLA[3]}" >> $DIR_REGISTRO/$EJECUCION/$FECHA.$HORA.$INFORME
	fi
	echo "Script ejecutado: ${DATOS_PLANTILLA[4]}" >> $DIR_REGISTRO/$EJECUCION/$FECHA.$HORA.$INFORME
	echo "E-mail de notificacion: ${DATOS_PLANTILLA[5]}" >> $DIR_REGISTRO/$EJECUCION/$FECHA.$HORA.$INFORME
	echo "Directorio de registro: $DIR_REGISTRO/$EJECUCION" >> $DIR_REGISTRO/$EJECUCION/$FECHA.$HORA.$INFORME
	echo "Directorio de resultados: $DIR_RESULTADOS" >> $DIR_REGISTRO/$EJECUCION/$FECHA.$HORA.$INFORME

	#RESUMEN ESTADO DE LA EJECUCION
	echo -e "\n\n\n\t\t\t\t\t RESUMEN ESTADO ACTUAL" >> $DIR_REGISTRO/$EJECUCION/$FECHA.$HORA.$INFORME
	echo "Numero de equipos pendientes de ejecucion: $pendientes" >> $DIR_REGISTRO/$EJECUCION/$FECHA.$HORA.$INFORME
	echo "Numero de equipos donde se ha ejecutado el script: $ejecutadas" >> $DIR_REGISTRO/$EJECUCION/$FECHA.$HORA.$INFORME
	echo "Numero de equipos donde se ha ejecutado el script y se ha descargado el resultado de la ejecucion: $finalizadas" >> $DIR_REGISTRO/$EJECUCION/$FECHA.$HORA.$INFORME

	#DETALLES
	echo -e "\n\n\n\t\t\t\t\t RESULTADOS DE LA EJECUCION POR EQUIPOS" >> $DIR_REGISTRO/$EJECUCION/$FECHA.$HORA.$INFORME
	#Descomprimimos los resultados
	log "Descomprimiendo resultados ..."
	if [ -d $DIR_RESULTADOS ];then
		cd $DIR_RESULTADOS
		for i in `ls | grep .tar.gz`;do
			nombreficheroaux=`echo $i | sed 's/'.tar.gz'//g'`
			#ipaux=`echo $nombreficheroaux | grep --only-matching --perl-regex "(?<=$EJECUCION.$FECHA.$HORA.).*"`
			ipaux=`echo $nombreficheroaux | awk -F"." '{print $4"."$5"."$6"."$7}'`
			#echo "DEBUG: nombreficheroaux = $nombreficheroaux - ipaux = $ipaux"
        		tar -xzpf $i 2> /dev/null
			rm -rf $i
			for j in `ls $nombreficheroaux*`;do
				if [ `echo $j | grep "salida.log$"` ];then
					echo -e "\nIP: $ipaux" >> $DIR_REGISTRO/$EJECUCION/$FECHA.$HORA.$INFORME				
					echo "($j):" >> $DIR_REGISTRO/$EJECUCION/$FECHA.$HORA.$INFORME
					cat $j >> $DIR_REGISTRO/$EJECUCION/$FECHA.$HORA.$INFORME
					echo -e "\n" >> $DIR_REGISTRO/$EJECUCION/$FECHA.$HORA.$INFORME
					gestorBD "UPDATE ejecucionhost SET host_informe='$DIR_RESULTADOS/$j' WHERE ejecucion='$EJECUCION' AND host='$ipaux'"
				elif [ `echo $j | grep "saweso.log$"` ];then
					gestorBD "UPDATE ejecucionhost SET host_log='$DIR_RESULTADOS/$j' WHERE ejecucion='$EJECUCION' AND host='$ipaux'"		
				else
					gestorBD "UPDATE ejecucionhost SET host_resultados=CONCAT_WS(';', host_resultados, '$DIR_RESULTADOS/$j') WHERE ejecucion='$EJECUCION' AND host='$ipaux'"			
				fi
			done
		done
		log "Informe en texto plano generado con éxito: $DIR_REGISTRO/$EJECUCION/$FECHA.$HORA.$INFORME"
	else
		log "No existe el directorios de resultados: $DIR_REGISTRO/$EJECUCION/$FECHA.$HORA.$INFORME"
	fi
fi

gestorBD "UPDATE ejecucion SET informe='$DIR_REGISTRO/$EJECUCION/$FECHA.$HORA.$INFORME' WHERE codigo='$EJECUCION'"
log "Registramos informe finalizado en la ejecucion $EJECUCION"
}


#NOMBRE: Mail - Envia el informe generado por la funcion GenerarInforme al destinatario especificado en el array DATOS_PLANTILLA
#SINOPSIS: Mail
#SALIDA:
Mail()
{
local INFORME="informe.txt"
#Tamaño máximo que puede ocupar el correo electrónico de 9MB
tamano_maximo=9
tamano_informe=`du -m $DIR_REGISTRO/$EJECUCION/$FECHA.$HORA.$INFORME | awk '{print $1}'`

log "Enviando informe mediante e-mail"
#Comprobacion sistema de mensajería
mailx=`which mail | wc -l`
postfix=`dpkg -l | grep postfix | wc -l`
if [ $mailx -eq 0 ];then
        log "No se puede localizar el comando mail. Se cancela el envío de correo a ${DATOS_PLANTILLA[5]}"
elif [ $postfix -eq 0 ];then
	log "No esta instalado el paquete postfix. Se cancela el envío de correo a ${DATOS_PLANTILLA[5]}"
elif [ $tamano_informe -gt $tamano_maximo  ];then
	log "El tamaño de informe generado supera el tamaño máximo establecido ($tamano_maximo MB). Se cancela el envío de correo a ${DATOS_PLANTILLA[5]}"
else
	cat $DIR_REGISTRO/$EJECUCION/$FECHA.$HORA.$INFORME | mail -s "SAWESO: Ejecucion $EJECUCION" ${DATOS_PLANTILLA[5]}
	log "Informe notificado mediante e-mail a ${DATOS_PLANTILLA[5]}"
fi
}


############################################## FIN DECLARACION DE FUNCIONES ##############################################
################################################### CUERPO DEL SCRIPT ####################################################

#Verificamos si es posible ejecutar saweso
ComprobarEntornoEjecucion
ObtenerDatosIniciales

log "##### SAWESO.SH (`whoami`): INICIO FASES EJECUCION Y RESULTADOS DE SAWESO $FECHA.$HORA #####"
#Iniciamos la ejecucion y revisamos cuando ha finalidado
MultiHilo
CheckFinish
#Cerramos el tunel SSH dynamic port forwarding with SOCKS creado
case "${DATOS_PLANTILLA[2]}" in
        "equipo")
		CloseTunneling
	;;
esac
log "##### SAWESO.SH (`whoami`): FIN FASES EJECUCION Y RESULTADOS DE SAWESO $FECHA.$HORA #####"

gestorBD "UPDATE ejecucion SET estado='GENERANDO INFORME' WHERE codigo='$EJECUCION'"
log "Cambiamos estado de la ejecucion $EJECUCION a GENERANDO INFORME"
log "##### SAWESO.SH (`whoami`): INFORME Y NOTIFICACION DE SAWESO $FECHA.$HORA #####"
#Generamos el informe de la ejecucion
GenerarInforme

gestorBD "UPDATE ejecucion SET estado='ENVIANDO EMAIL' WHERE codigo='$EJECUCION'"
log "Cambiamos estado de la ejecucion $EJECUCION a ENVIANDO EMAIL"
#Reportamos el informe generado
Mail

gestorBD "UPDATE ejecucion SET estado='FINALIZADA' WHERE codigo='$EJECUCION'"
log "Cambiamos estado de la ejecucion $EJECUCION a FINALIZADA"
log "##### SAWESO.SH (`whoami`): FIN DE SAWESO $FECHA.$HORA #####"
log "################################################################################################################"

################################################### FIN CUERPO DEL SCRIPT ####################################################
